/*
  -------------------- file teval.h
  Created 13 Jul 94 by Andrew Shapira, Rensselaer Polytechnic Institute.

	Copyright (C) 1994 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*	Note: "teval" stands for "tuple evaluation".
*/

#ifndef _130794_194920
#define _130794_194920

#include "coord.h"
#include "tmask.h"
#include "../libinclude/ntup.h"

#define DEBUG_EVAL

/*  Note: if you change this, change teval_copy().
*/
struct teval_struct {
  int pslen;      /* length of partial solution used to compute this */
  int *pvals;     /* [pos ch: 0 .. s_nposchars-1] */
  int **qvecs;    /* [q value: 0 .. QVEC_ROWS-1][neg ch: 0 .. s_nnegchars-1] */
  unsigned tiebreak;/* a pseudo-random number used for tiebreaks, */
    /* based solely on the vector of piidc's that form the partial solution */
  unsigned *ps;   /* used so that there are never any ties */
};

int can_determine_max P(( const struct coord_struct * const ));
void teval_alloc P(( struct teval_struct * const, void  *(*) P((const int)) ));
void teval_copy P(( struct teval_struct * const, const struct teval_struct * ));
int do_tupdate P(( const int, const int ));
void do_teval P(( struct teval_struct * const ));
int teval_from_scratch
  P(( const int * const, const int, struct teval_struct * const ));
int teval_compare P(( const struct teval_struct * const ,
                      const struct teval_struct * const ));
void teval_setup_pq P(( const int, const int ));
void teval_setup_invariants P((
  const int, const struct tmask_struct * const,
  const struct tmask_struct * const, void *(*) P((const int)) ));

#ifdef DEBUG_EVAL
void debug_teval P(( const char * const, const struct teval_struct * const ));
#endif /*SHOW_EVAL*/


#endif /* _130794_194920 */
