/*
  -------------------- file tmask.h
  Created 17 May 94 by Andrew Shapira, Rensselaer Polytechnic Institute.

	Copyright (C) 1994 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _170594_220245
#define _170594_220245

#include "../prog/bitarray.h"
#include "../libinclude/ntup.h"
#include "coord.h"

struct tmask_struct {
  /* Dimension information
  */
  int nchars;           /* characters */
  int *ntrans;          /* for each character, number of translations */
  int *ntrans_words;    /* ntrans_words[i] = BIT_WORDS(ntrans[i]) */

  /* copy (not merely pointer) of pi that was used to make
     this tmask structure
  */
  struct coord_struct *pi;

  /*  mask[piid][c][ch] gives mask of which translations match.
      piid is pi id (square number) (0 .. npi_ids-1)
      c is 0 or 1 (0=black, 1=white)
      ch is 0 .. nchars-1.
      d is 0 .. ntrans[ch]
      bit d of mask[piid][c][ch] is 1 iff
        square that piid translates to under d is color c in character ch
      Note: the range of d is 0 .. (ntrans[ch]-1).  (The number of
      translations depends on the character.)
  */
  BITWORD ****mask;
};

#ifdef DEBUG
void debug_tmask P(( const struct tmask_struct *));
#endif  /*DEBUG*/

void tmask_make P(( const struct character_struct * const, const int,
  const struct coord_struct *const, const int,
  struct tmask_struct *const, void *(*) P((const int)) ));
int tmask_pos_satisfied P(( const struct pix_struct * const, const int,
  const int, const struct tmask_struct * const ));
int tmask_neg_satisfied P(( const struct pix_struct * const, const int,
  const int, const struct tmask_struct * const ));


#endif /* _170594_220245 */
