/*
  -------------------- file random.h
  Created 14 May 94 by Andrew Shapira, Rensselaer Polytechnic Institute.

	Copyright (C) 1994 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _140594_171429
#define _140594_171429

#include "common.h"

unsigned environ_rand P(( void ));

void srand48 P(( long ));
long lrand48 P(( void ));

/**************************************************************************
  inline functions (be careful about changing the order of func. definition)
**************************************************************************/

/*  declarations (included for compatibility with non-gnuc compilers) */
#ifndef __GNUC__
int coinflip_rat P(( const unsigned long, const unsigned long ));
#endif  /*__GNUC__*/

/*  note: inline is defined to the empty string unless we're using gnuc */
#undef  INLINE
#ifdef  SRCFILE_LIBPROG_RANDOM
#define INLINE  __inline__
#else
#define INLINE  __inline__ extern
#endif

#if defined(__GNUC__) || defined(SRCFILE_LIBPROG_RANDOM)

/*  Perform a Bournoulli trial.  (A success/failure trial.)
    Given a probability r/s, perform an experiment that "succeeds"
    r/s of the time.

    Returns 1 with probability r/s.
    Returns 0 with probability 1 - r/s.

    It is a fatal error to have s==0, or r/s > 1, or s > 2^31.

    Notes:
      (1) r and s need not be in lowest terms.
      (2) The restriction s <= 2^31 is a peculiarity of the particular
          method used to implement this function.  Conceivably, at some
          future date, this function could be changed to remove this
          restriction.  It is related to the fact that random(3) generates
          integers in the range 0 .. 2^31 - 1.
      (3) This function does only integer arithmetic.  There is no
          rounding error.  The only source of error is in the random
          number generator.
      (4) This function may or may not call any given random number
          generator, e.g. random().
      (5) Currently, random(3) is used to generate the random number, when
          one is generated.
  
  Written by Andrew Shapira, Rensselaer Polytechnic Institute, May 1994.
*/
INLINE int
coinflip_rat(
const unsigned long r,
const unsigned long s
)
{
  extern long random();

  if (s==0)  fatal("coinflip_rat: div. by 0.");
  if (r>s)   fatal("coinflip_rat: prob. > 1.");
  if (r==s)  return(1);
  if (r==0)  return(0);

  return((random() % s) < r);
}

#endif  /* defined(__GNUC__) || defined(source file) */

#endif /* _140594_171429 */
