/*
  -------------------- file timer.c
  Created 11 Mar 92 by Andrew Shapira, Rensselaer Polytechnic Institute.
  Routines for timing the execution of code segments.  Uses
  gettimeofday() and is not super exact.

	Copyright (C) 1992 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <sys/time.h>

#include "our_common.h"
#include "timer.h"
#include "util.h"

/*  Start an interval timer.
*/
void
begin_timing(
struct timeval *p
)
{
	int gettimeofday P((struct timeval *, void *));

  if (gettimeofday(p,NULL))
    fatal("begin_timing: gettimeofday");
}

void
end_timing2(
const struct timeval  *start,
const struct timeval  *end,
long                  *secondsret,
long                  *microsecondsret
)
{
  *secondsret = end->tv_sec - start->tv_sec;
  *microsecondsret = end->tv_usec - start->tv_usec;
  time_normalize(secondsret,microsecondsret);
}

void
end_timing(
const struct timeval  *p1,
long                  *secondsret,
long                  *microsecondsret
)
{
  struct timeval p2;
	int gettimeofday P((struct timeval *, void *));

  gettimeofday(&p2, NULL);
  end_timing2(p1,&p2,secondsret,microsecondsret);
}

void
time_normalize(
long *secondsret,
long *microsecondsret
)
{
  register long   s,us;

  s = *secondsret;
  us = *microsecondsret;
  while ((us < 0) && (s > 0)) {
    s--;
    us += 1000000;
  }
  while (us >= 1000000) {
    s++;
    us -= 1000000;
  }
  *secondsret = s;
  *microsecondsret = us;
}
